ARGS_QA_TRAIN = {
    'learning_rate': 5e-5,
    'weight_decay': 0,
    'adam_beta1': 0.9,
    'adam_beta2': 0.999,
    'adam_epsilon': 1e-8,
    'max_grad_norm':  1.0,
    'num_train_epochs': 3.0,

}

# maybe implement later
# 'max_steps': -1,
# 'warmup_steps': 0,
# # 'logging_dir':  #todo
# 'logging_first_step': False,
# 'logging_steps': 500,
# 'save_steps': 500,
#  #'save_total_limit':
# 'no_cuda': False,
# 'seed': 42,
# 'fp16': False,
# 'fp16_opt_level': "O1",
# 'local_rank': -1,
# #'tpu_num_cores':,
# "debug": False,
#
